import 'package:flutter/material.dart';

import 'package:fastyle_exchanges/fastyle_exchanges.dart';
import 'package:fastyle_dart/fastyle_dart.dart';

const _kHeaderPadding = 16.0;
const _kContentPadding = EdgeInsets.symmetric(vertical: _kHeaderPadding);

class FastExchangeHomePage extends StatefulWidget {
  ///
  /// The padding for the page.
  ///
  final EdgeInsetsGeometry contentPadding;

  ///
  /// A button displayed floating above body, in the bottom right corner.
  ///
  final Widget? floatingActionButton;

  ///
  /// The widgets below the app bar in the tree.
  ///
  final List<Widget> children;

  ///
  /// The size of the app bar when it is fully expanded.
  ///
  final double appBarExpandedHeight;

  ///
  /// A list of Widgets to display in a row after the title.
  ///
  final List<Widget>? actions;

  ///
  /// Text displayed below the title.
  /// Typically, a description of the current contents of the app.
  ///
  final String? subtitleText;

  ///
  /// Title of the app.
  ///
  final String? titleText;

  ///
  /// A widget to display before the toolbar's title.
  ///
  final Widget? leading;

  final LinearGradient? appBarBackgroundLinearGradient;

  final Color? appBarBackgroundColor;

  final Widget? appBarDecoration;

  FastExchangeHomePage({
    Key? key,
    required this.children,
    this.appBarExpandedHeight = kFastExpandedHeight,
    this.contentPadding = _kContentPadding,
    this.appBarBackgroundLinearGradient,
    this.appBarBackgroundColor,
    this.floatingActionButton,
    this.appBarDecoration,
    this.subtitleText,
    this.titleText,
    this.actions,
    this.leading,
  })  : assert(
          appBarExpandedHeight >= kFastExpandedHeight ? true : false,
        ),
        super(key: key);

  @override
  _FastExchangeHomePageState createState() => _FastExchangeHomePageState();
}

class _FastExchangeHomePageState extends State<FastExchangeHomePage>
    with TickerProviderStateMixin {
  //TODO: Stop animation when not visible anymore
  late final AnimationController _controller = AnimationController(
    duration: const Duration(seconds: 2),
    vsync: this,
  )..repeat(reverse: true);

  late final Animation<double> _animation = CurvedAnimation(
    parent: _controller,
    curve: Curves.easeInOut,
  );

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final whiteColor = ThemeHelper.colors.getWhiteColor(context);

    return FastHomePageLayout(
      appBarBackgroundLinearGradient: widget.appBarBackgroundLinearGradient,
      appBarBackgroundColor: widget.appBarBackgroundColor,
      appBarExpandedHeight: widget.appBarExpandedHeight,
      appBarDecoration: CustomPaint(
        painter: GraphAppBarPainter(
          backgroundColor: whiteColor,
          borderColor: whiteColor,
          cursorColor: whiteColor,
          backgroundOpacity: 0.15,
          borderOpacity: 0.65,
        ),
        foregroundPainter: AnimatedGraphCursorPainter(animation: _animation),
      ),
      contentPadding: widget.contentPadding,
      subtitleText: widget.subtitleText,
      titleText: widget.titleText,
      actions: widget.actions,
      leading: widget.leading,
      floatingActionButton: widget.floatingActionButton,
      children: widget.children,
    );
  }
}
