import 'package:flutter/material.dart';

class AnimatedGraphCursorPainter extends CustomPainter {
  final Animation<double> animation;
  final double maxRadius;
  final double minRadius;
  final double opacity;
  final Color color;

  AnimatedGraphCursorPainter({
    required this.animation,
    this.color = Colors.white,
    this.opacity = 0.25,
    this.maxRadius = 16,
    this.minRadius = 6,
  }) : super(repaint: animation);

  @override
  void paint(Canvas canvas, Size size) {
    _paintAnimatedCursor(canvas, size);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => true;

  void _paintAnimatedCursor(Canvas canvas, Size size) {
    final animationPercent = animation.value;
    final distance = maxRadius - minRadius;
    final x = size.width * 0.815;
    final y = size.height * 0.45;
    final center = Offset(x, y);
    final paint = Paint();

    paint.color = color.withOpacity(opacity);
    canvas.drawCircle(center, minRadius + distance * animationPercent, paint);
  }
}
